/*
THR_EST.PRC

written by:

Bruce E. Hansen
Department of Economics
Boston College
Chestnut Hill, MA 02167
(617) 552-3678
bruce.hansen@bc.edu
http:/fmwww.bc.edu/EC-V/Hansen.fac.html


This is a GAUSS procedure.  It computes estimates and confidence
intervals for threshold models.  The procedure takes the form

{qhat,sse} =  = thr_est(dat,names,yi,xi,qi,h)

The inputs are:
  dat     = data matrix (nxk)
  names   = variable names (kx1), corresponding to dat matrix
  yi      = index of dependent (y) variable, e.g.: yi = 1
  xi      = indexes of independent (x) variables, e.g.: xi = 2|3
  qi      = index of threshold (q) variable, e.g.: qi = 4;
  h       = heteroskedasticity indicator
            Set h=0 to impose homoskedasticity assumption
            Set h=1 to use White-correction for heteroskedasticity

Outputs:
  qhat    = LS estimate of threshold
  sse     = minimizing sum of squared errors

The remaining outputs are printed to the screen.

There is one global variable:

_graph   = 0;     @ Graph indicator
                    Set _graph=0 to not view the graph of the likelihood
                    Set _graph=1 to view the graph of the likelihood and print output @


Notes:
  (1)  Do not include a constant in the independent variables;
       the program automatically adds an intercept to the regression.

  (2)  There are three other control parameters, governing the choice
       of confidence level, the nonparametric method used to compute
       the nuisance parameter in the event of heteroskedastic, and
       whether to print the graph of the likelihood.  These controls
       are listed at the beginning of the procedure code.

  (3)  The program includes code to produce a graph.
       This requires that the pgraph library is active.
       Thus, the command "library pgraph" must have been issued
       previously in the GAUSS session.
       If this is a problem, delete the lines between
          "if _graph==1;"  and  "endif;"



Example:
If the nxk matrix "dat" contains the dependent variable in the first
column, the independent variables in the second through tenth columns,
and the threshold variable in the fifth.  The vector "names" contains
the names of all variables in "dat".

    xi = 2|3|4|5|6|7|8|9|10;
    {qhat,sse} = thr_est(dat,names,1,xi,5,0);

If the error is (possibly) heteroskedatic, the last "0"
should be replaced with a "1".


*******************************************************************/

proc (2) = thr_est(dat,names,yi,xi,qi,h);

local n,qs,q,y,x,k,yname,xname,qname,mi,beta,e,ee,sig,xe,se,vy,r_2,
qn,sn,irb,mm,sume,ci,r,irf,ir,mmi,rmin,smin,qhat,sighat,
i1,i2,x1,y1,x2,y2,mi1,mi2,beta1,beta2,e1,e2,ej,n1,n2,ee1,ee2,sig1,sig2,
sig_jt,se1,se2,vy1,vy2,r2_1,r2_2,r2_joint,lr,r1,r2,qx,qh,m1,m2,g1,g2,
sigq,hband,u,u2,f,df,eps,sige,kh,eta2,c1,c2,lr1,lr2,qhat1,qhat2,z,
beta1l,beta1u,beta2l,beta2u,b1,b2,ser1,ser2,ff,p,tit,vtit,clr,plr,
_conf1,_conf2,_nonpar;

@ Control Parameters, can be modified if desired  @

_conf1 = .95;  @ Confidence Level for Confidence Regions  @
_conf2 = .8;   @ Confidence Level for first step of two-step
                 Confidence Regions for regression parameters @
_nonpar = 2;   @ Indicator for non-parametric method used to
                 estimate nuisance scale in the presence of
                 heteroskedasticity (only relevant if h=1).
                 Set _nonpar=1  to estimate regressions using
                 a quadratic.
                 Set _nonpar=2  to estimate regressions using
                 an Epanechnikov kernel with automatic bandwidth. @


if (h .ne 0)*(h .ne 1);
"You have entered h = " h;
"This number must be either 0 (homoskedastic case) or 1 (heteoskedastic)";
"The program will either crash or produce invalid results";
endif;
if (_nonpar .ne 1)*(_nonpar .ne 2)*(h==1);
"You have entered _nonpar = " _nonpar;
"This number should be either 1 (quadratic regression)"
"   or 2 (kernel regression)";
"The program will employ the quadratic regression method";"";"";
endif;

n = rows(dat);
q = dat[.,qi];
qs = sortind(q);
q = q[qs];
y = dat[qs,yi];
x = ones(n,1)~dat[qs,xi];
k = cols(x);
yname = names[yi];
qname = names[qi];
xname = "Constant"|names[xi];

mi = invpd(moment(x,0));
beta = mi*(x'y);
e=y-x*beta;
ee=e'e;
sig = ee/(n-k);
xe = x.*e;
if h==0;
  se = sqrt(diag(mi)*sig);
else;
  se = sqrt(diag(mi*moment(xe,0)*mi));
endif;
vy = sumc((y - meanc(y)).^2);
r_2 = 1-ee/vy;

qs = unique(q,1);
qn = rows(qs);
sn = zeros(qn,1);

irb = zeros(n,1);
mm = zeros(k,k);
sume = zeros(k,1);
ci = 0;

r=1; do while r<=qn;
  irf = (q .<= qs[r]);
  ir = irf - irb;
  irb = irf;
  ci = ci + sumc(ir);
  mm = mm + moment(selif(x,ir),0);
  sume = sume + sumc(selif(xe,ir));
  mmi = mm - mm*mi*mm;
  if (ci .> k+1)*(ci .< (n-k-1));
    sn[r] = ee - sume'(sume/mmi);
  else;
    sn[r]=ee;
  endif;
r=r+1;endo;

rmin=minindc(sn);
smin=sn[rmin];
qhat=qs[rmin];
sighat=smin/n;

i1 = (q .<= qhat);
i2 = 1-i1;
x1 = selif(x,i1);
y1 = selif(y,i1);
x2 = selif(x,i2);
y2 = selif(y,i2);
mi1 = invpd(moment(x1,0));
mi2 = invpd(moment(x2,0));
beta1 = mi1*(x1'y1);
beta2 = mi2*(x2'y2);
e1 = y1 - x1*beta1;
e2 = y2 - x2*beta2;
ej = e1|e2;
n1 = rows(y1);
n2 = rows(y2);
ee1 = e1'e1;
ee2 = e2'e2;

if _graph==1;

sig1 = ee1/(n1-k);
sig2 = ee2/(n2-k);
sig_jt = (ee1+ee2)/(n-k*2);
if h==0;
  se1 = sqrt(diag(mi1)*sig_jt);
  se2 = sqrt(diag(mi2)*sig_jt);
else;
  se1 = sqrt(diag(mi1*moment(x1.*e1,0)*mi1));
  se2 = sqrt(diag(mi2*moment(x2.*e2,0)*mi2));
endif;
vy1 = sumc((y1 - meanc(y1)).^2);
vy2 = sumc((y2 - meanc(y2)).^2);
r2_1 = 1 - ee1/vy1;
r2_2 = 1 - ee2/vy2;
r2_joint = 1 - (ee1+ee2)/vy;

if h==0;
  lr  = (sn-smin)/sighat;
elseif h==1;
  r1 = (x*(beta1-beta2)).^2;
  r2 = r1.*(ej.^2);
  qx = q.^(0~1~2);
  qh = qhat.^(0~1~2);
  m1 = r1/qx;
  m2 = r2/qx;
  g1 = qh*m1;
  g2 = qh*m2;
  if _nonpar==2;
    sigq = sqrt(meanc((q-meanc(q)).^2));
    hband = 2.344*sigq/(n^(.2));
    u = (qhat-q)/hband;
    u2 = u.^2;
    f = meanc((1-u2).*(u2.<=1))*(.75/hband);
    df = -meanc(-u.*(u2.<=1))*(1.5/(hband^2));
    eps = r1 - qx*m1;
    sige = (eps'eps)/(n-3);
    hband = sige/(4*f*((m1[3]+(m1[2]+2*m1[3]*qhat)*df/f)^2));
    u2 = ((qhat-q)/hband).^2;
    kh = ((1-u2)*.75/hband).*(u2.<=1);
    g1 = meanc(kh.*r1);
    g2 = meanc(kh.*r2);
  endif;
  eta2 = g2/g1;
  lr = (sn-smin)/eta2;
endif;
c1 = -2*ln(1-sqrt(_conf1));
c2 = -2*ln(1-sqrt(_conf2));
lr1 = (lr .>= c1);
lr2 = (lr .>= c2);
if maxc(lr1)==1;
  qhat1 = qs[minindc(lr1)];
  qhat2 = qs[qn+1-minindc(rev(lr1))];
else;
  qhat1 = qs[1];
  qhat2 = qs[qn];
endif;
z = maxindc((cdfn(seqa(.01,.01,300))*2-1) .>= _conf1)/100;
beta1l = beta1 - se1*z;
beta1u = beta1 + se1*z;
beta2l = beta2 - se2*z;
beta2u = beta2 + se2*z;
r=1; do while r<=qn;
  if lr2[r]==0;
    i1 = (q .<= qs[r]);
    x1 = selif(x,i1);
    y1 = selif(y,i1);
    trap 1;
    mi1 = invpd(moment(x1,0));
    trap 0;
    if scalerr(mi1)==0;
      b1 = mi1*(x1'y1);
      e1 = y1 - x1*b1;
      if h==0;
        ser1 = sqrt(diag(mi1)*(e1'e1)/(rows(y1)-k));
      else;
        ser1 = sqrt(diag(mi1*moment(x1.*e1,0)*mi1));
      endif;
      beta1l = minc(beta1l'|(b1 - ser1*z)');
      beta1u = maxc(beta1u'|(b1 + ser1*z)');
    endif;
    i2 = 1-i1;
    x2 = selif(x,i2);
    y2 = selif(y,i2);
    trap 1;
    mi2 = invpd(moment(x2,0));
    trap 0;
    if scalerr(mi2)==0;
      b2 = mi2*(x2'y2);
      e2 = y2 - x2*b2;
      if h==0;
        ser2 = sqrt(diag(mi2)*(e2'e2)/(rows(y2)-k));
      else;
        ser2 = sqrt(diag(mi2*moment(x2.*e2,0)*mi2));
      endif;
      beta2l = minc(beta2l'|(b2 - ser2*z)');
      beta2u = maxc(beta2u'|(b2 + ser2*z)');
    endif;
  endif;
r=r+1;endo;

ff = "#*.*lG"~20~8;
"Global OLS Estimation, Without Threshold";
print ("Dependent Variable:     " $+ yname);
if h==1;
  "Heteroskedasticity Correction Used";
elseif h==0;
  "OLS Standard Errors Reported";
endif;
"";
p = printfm("Variable"~"Estimate"~"St Error",0~0~0,ff);"";
"       __________________________________________";
p = printfm(xname~beta~se,0~1~1,ff);"";
"";
"Observations                      " n;
"Degrees of Freedom                " (n-k);
"Sum of Squared Errors             " ee;
"Residual Variance                 " sig;
"R-squared                         " r_2;
"Heteroskedasticity Test (P-Value) " het_test(e,x);
"";"";
"__________________________________________________";
"";"";
"Threshold Estimation";
"";
tit =  "Threshold Variable =      " $+ qname;
print tit;
"Threshold Estimate        " qhat;
tit = "" $+ ftocv(_conf1*100,2,0) $+ "% Confidence Interval:  [ ";
tit = "" $+ tit $+ ftocv(qhat1,8,8) $+ " , ";
tit = "" $+ tit $+ ftocv(qhat2,8,8) $+ " ]";
print tit;
"";"";
"Sum of Squared Errors             " (ee1+ee2);
"Residual Variance                 " sig_jt;
"Joint R-Squared                   " r2_joint;
"Heteroskedasticity Test (p-value) " het_test(ej,x);
"";"";
"     _____________________________________________";
"";"";
tit = "Regime 1:    " $+ qname $+ " <= " $+ ftocv(qhat,8,8);
print tit;
"";
"                  Parameter Estimates";"";
p = printfm("Variable"~"Estimate"~"St Error",0~0~0,ff);"";
"        ________________________________________";
p = printfm(xname~beta1~se1,0~1~1,ff);"";
"";
vtit = "          " $+ ftocv(_conf1*100,2,0)
$+ "% Confidence Regions for Parameters";
print vtit; "";
p = printfm("Variable"~"Low"~"High",0~0~0,ff);"";
"        ________________________________________";
p = printfm(xname~beta1l~beta1u,0~1~1,ff);"";
"";
"Observations                      " n1;
"Degrees of Freedom                " (n1-k);
"Sum of Squared Errors             " ee1;
"Residual Variance                 " sig1;
"R-squared                         " r2_1;
"";"";
"     _____________________________________________";
"";"";
tit = "Regime 2:    " $+ qname $+ " > " $+ ftocv(qhat,8,8);
print tit;
"";
"                  Parameter Estimates";"";
p = printfm("Variable"~"Estimate"~"St Error",0~0~0,ff);"";
"        ________________________________________";
p = printfm(xname~beta2~se2,0~1~1,ff);"";
"";
print vtit; "";
p = printfm("Variable"~"Low"~"High",0~0~0,ff);"";
"        ________________________________________";
p = printfm(xname~beta2l~beta2u,0~1~1,ff);"";
"";
"Observations                      " n2;
"Degrees of Freedom                " (n2-k);
"Sum of Squared Errors             " ee2;
"Residual Variance                 " sig2;
"R-squared                         " r2_2;
"";"";
"__________________________________________________";

   graphset;
   _pdate = "";
   fonts("complex simgrma");
   title("Confidence Interval Construction for Threshold");
   ylabel("Likelihood Ratio Sequence in \202g\201");
   tit = "Threshold Variable: " $+ qname;
   xlabel(tit);
   _pxsci = 8;
   _pysci = 8;
   clr = ones(qn,1)*c1;
   _plegstr="" $+ "LR]n[(\202g\201)\000" $+ ftocv(_conf1*100,2,0)
   $+ "% Critical";
   _plegctl = 1;
   xy(qs,lr~clr);
endif;

retp(qhat,ee1+ee2);
endp;

proc het_test(e,x);
local e2,x2,v,te;
  e2 = e.^2;
  x2 = x.^2;
  v = e2 - x2*(e2/x2);
  e2 = e2 - meanc(e2);
  te = rows(e)*(1-(v'v)/(e2'e2));
retp(cdfchic(te,cols(x)));
endp;

